﻿#if DOTNET35
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using NUnit.Framework;

using MiscUtil.Linq;
using MiscUtil.Linq.Extensions;

namespace MiscUtil.UnitTests.Collections
{
    [TestFixture]
    public class OrderedDataProducerTest
    {
        class CompliantProducer<T> : IDataProducer<T>
        { // no error checking!
            public void Produce(T value)
            {
                if (DataProduced != null) DataProduced(value);
            }
            public void ProduceAndEnd(params T[] values)
            {
                foreach (T value in values)
                {
                    Produce(value);
                }
                End();
            }
            public event Action<T> DataProduced;
            public void End()
            {
                if (EndOfData != null) EndOfData();
            }
            public event Action EndOfData;
        }

        [Test]
        public void OrderedProducerNullComparer()
        {
            var producer = new DataProducer<int>();
            var ordered = new OrderedDataProducer<int>(producer, null).ToList();
            producer.ProduceAndEnd(1, 3, 0, 2, 4);
            for (int i = 0; i < 5; i++)
            {
                Assert.AreEqual(i, ordered[i]);
            }
        }

        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void NullProducer()
        {
            var ordered = new OrderedDataProducer<int>(null, null);
        }

        [Test, ExpectedException(typeof(InvalidOperationException))]
        public void EndTwice()
        {
            var producer = new CompliantProducer<int>();
            var ordered = new OrderedDataProducer<int>(producer, null).ToList();
            producer.ProduceAndEnd(1, 3, 0, 2, 4);
            for (int i = 0; i < 5; i++)
            {
                Assert.AreEqual(i, ordered[i], "verify working first");
            }
            producer.End(); // boom
        }

        [Test, ExpectedException(typeof(InvalidOperationException))]
        public void ProduceAfterEnd()
        {
            var producer = new CompliantProducer<int>();
            var ordered = new OrderedDataProducer<int>(producer, null).ToList();
            producer.ProduceAndEnd(1, 3, 0, 2, 4);
            for (int i = 0; i < 5; i++)
            {
                Assert.AreEqual(i, ordered[i], "verify working first");
            }
            producer.Produce(6); // boom
        }
    }
}
#endif