﻿using System;
using System.Collections.Generic;
using MiscUtil.Extensions;
namespace MiscUtil.Linq
{
    /// <summary>
    /// Comparer to sort elements of T based on a projection
    /// </summary>
    /// <typeparam name="T">Element type</typeparam>
    /// <typeparam name="TKey">The projected type (usually a property)</typeparam>
    internal class SelectorComparer<T, TKey> : IComparer<T>
    {
        private readonly Func<T, TKey> selector;
        private readonly IComparer<TKey> comparer;
        private readonly bool descending;

        /// <summary>
        /// Create a new comparer based on a projection
        /// </summary>
        /// <param name="selector">The projection function to apply to obtain the key to compare</param>
        /// <param name="comparer">The comparer used to compare keys (defaults if null)</param>
        /// <param name="descending">Should the order be considered ascending or descending?</param>
        public SelectorComparer(
            Func<T, TKey> selector,
            IComparer<TKey> comparer,
            bool descending)
        {
            selector.ThrowIfNull("selector");
            this.selector = selector;
            this.comparer = comparer ?? Comparer<TKey>.Default;
            this.descending = descending;
        }

        int IComparer<T>.Compare(T x, T y)
        {
            return descending
                ? comparer.Compare(selector(y), selector(x))
                : comparer.Compare(selector(x), selector(y));
        }
    }

}
